@@ -20,7 +20,10 @@ package MongoDB::BSON;
 # ABSTRACT: Tools for serializing and deserializing data in BSON form
 
 use version;
-our $VERSION = 'v0.703.5'; # TRIAL
+our $VERSION = 'v0.999.998.1'; # TRIAL
+
+use XSLoader;
+XSLoader::load("MongoDB", $VERSION);
 
 use Moose;
 use namespace::clean -except => 'meta';
@@ -108,6 +111,34 @@ $MongoDB::BSON::use_boolean = 0;
 
 $MongoDB::BSON::use_binary = 0;
 
+sub decode_bson {
+    my ($msg,$client) = @_;
+    my @decode_args;
+    if ( $client ) {
+        @decode_args = map { $client->$_ } qw/dt_type inflate_dbrefs inflate_regexps/;
+        push @decode_args, $client;
+    }
+    else {
+        @decode_args = (undef, 0, 0, undef);
+    }
+    my $struct = eval { MongoDB::BSON::_decode_bson($msg, @decode_args) };
+    Carp::confess($@) if $@;
+    return $struct;
+}
+
+sub encode_bson {
+    my ($struct, $clean_keys, $max_size) = @_;
+    $clean_keys = 0 unless defined $clean_keys;
+    my $bson = eval { MongoDB::BSON::_encode_bson($struct, $clean_keys) };
+    Carp::confess($@) if $@;
+
+    if ( $max_size && length($bson) > $max_size ) {
+        Carp::confess("Document exceeds maximum size $max_size");
+    }
+
+    return $bson;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -124,7 +155,7 @@ MongoDB::BSON - Tools for serializing and deserializing data in BSON form
 
 =head1 VERSION
 
-version v0.703.5
+version v0.999.998.1
 
 =head1 NAME
 
@@ -195,11 +226,7 @@ you would like to have it deserialized as instances of L<MongoDB::BSON::Binary>
 
 =item *
 
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
+David Golden <david@mongodb.com>
 
 =item *
 
@@ -207,7 +234,11 @@ Mike Friedman <friedo@mongodb.com>
 
 =item *
 
-David Golden <david.golden@mongodb.org>
+Kristina Chodorow <kristina@mongodb.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
 
 =back
 
