@@ -2,7 +2,7 @@ package MongoDB::BSON::Regexp;
 # ABSTRACT: Regular expression type
 
 use version;
-our $VERSION = 'v0.703.5'; # TRIAL
+our $VERSION = 'v0.999.998.1'; # TRIAL
 
 use Moose;
 use namespace::clean -except => 'meta';
@@ -60,7 +60,7 @@ MongoDB::BSON::Regexp - Regular expression type
 
 =head1 VERSION
 
-version v0.703.5
+version v0.999.998.1
 
 =head1 AUTHORS
 
@@ -68,11 +68,7 @@ version v0.703.5
 
 =item *
 
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
+David Golden <david@mongodb.com>
 
 =item *
 
@@ -80,7 +76,11 @@ Mike Friedman <friedo@mongodb.com>
 
 =item *
 
-David Golden <david.golden@mongodb.org>
+Kristina Chodorow <kristina@mongodb.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
 
 =back
 
