@@ -19,9 +19,10 @@ package MongoDB::CommandResult;
 # ABSTRACT: MongoDB generic command result document
 
 use version;
-our $VERSION = 'v0.703.5'; # TRIAL
+our $VERSION = 'v0.999.998.1'; # TRIAL
 
 use Moose;
+use MongoDB::Error;
 use namespace::clean -except => 'meta';
 
 with 'MongoDB::Role::_LastError';
@@ -38,6 +39,18 @@ has result => (
     required => 1,
 );
 
+#pod =attr address
+#pod
+#pod Address ("host:port") of server that ran the command
+#pod
+#pod =cut
+
+has address => (
+    is       => 'ro',
+    isa      => 'HostAddress',
+    required => 1,
+);
+
 #pod =method last_errmsg
 #pod
 #pod Error string (if any) or the empty string if there was no error.
@@ -52,6 +65,30 @@ sub last_errmsg {
     return "";
 }
 
+sub assert {
+    my ($self) = @_;
+    if ( ! $self->result->{ok} ) {
+        my $err = $self->last_errmsg;
+        my $code = $self->result->{code};
+        my $error_class;
+
+        # XXX should we be detecting write/writeConcern/etc errors here?
+        if ( $err =~ /^(?:not master|node is recovering)/ ) {
+            $error_class = "MongoDB::NotMasterError";
+        }
+        else {
+            $error_class = "MongoDB::DatabaseError";
+        }
+
+        $error_class->throw(
+            result => $self,
+            ( length($err)   ? ( message => $err )  : () ),
+            ( defined($code) ? ( code    => $code ) : () ),
+        );
+    }
+    return 1;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -68,7 +105,7 @@ MongoDB::CommandResult - MongoDB generic command result document
 
 =head1 VERSION
 
-version v0.703.5
+version v0.999.998.1
 
 =head1 DESCRIPTION
 
@@ -81,6 +118,10 @@ available from the C<result> attribute of C<MongoDB::DatabaseError>.
 
 Hash reference with the result of a database command
 
+=head2 address
+
+Address ("host:port") of server that ran the command
+
 =head1 METHODS
 
 =head2 last_errmsg
@@ -93,11 +134,7 @@ Error string (if any) or the empty string if there was no error.
 
 =item *
 
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
+David Golden <david@mongodb.com>
 
 =item *
 
@@ -105,7 +142,11 @@ Mike Friedman <friedo@mongodb.com>
 
 =item *
 
-David Golden <david.golden@mongodb.org>
+Kristina Chodorow <kristina@mongodb.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
 
 =back
 
