@@ -19,7 +19,7 @@ package MongoDB::DBRef;
 # ABSTRACT: Native DBRef support
 
 use version;
-our $VERSION = 'v0.703.5'; # TRIAL
+our $VERSION = 'v0.999.998.1'; # TRIAL
 
 use Carp 'croak';
 use Tie::IxHash;
@@ -117,7 +117,7 @@ MongoDB::DBRef - Native DBRef support
 
 =head1 VERSION
 
-version v0.703.5
+version v0.999.998.1
 
 =head1 SYNOPSIS
 
@@ -193,11 +193,7 @@ database or collection concerned does not exist.
 
 =item *
 
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
+David Golden <david@mongodb.com>
 
 =item *
 
@@ -205,7 +201,11 @@ Mike Friedman <friedo@mongodb.com>
 
 =item *
 
-David Golden <david.golden@mongodb.org>
+Kristina Chodorow <kristina@mongodb.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
 
 =back
 
