@@ -20,7 +20,7 @@ package MongoDB::GridFS;
 # ABSTRACT: A file storage utility
 
 use version;
-our $VERSION = 'v0.703.5'; # TRIAL
+our $VERSION = 'v0.999.998.1'; # TRIAL
 
 use MongoDB::GridFS::File;
 use DateTime 0.78; # drops dependency on bug-prone Math::Round
@@ -121,15 +121,8 @@ sub _build_chunks {
 # If they are not found, they will be created.
 sub BUILD {
     my ($self) = @_;
-   
-    # check for the required indexes in the system.indexes collection
-    my $count = $self->_database->get_collection('system.indexes')->count({key=>{filename => 1}});
-    $count   += $self->_database->get_collection('system.indexes')->count({key=>{files_id => 1, n => 1}});
-    
-    # if we don't have the required indexes, create them now.
-    if ($count < 2){
-       $self->_ensure_indexes();
-    }
+    $self->_ensure_indexes();
+    return;
 }
 
 
@@ -433,7 +426,7 @@ MongoDB::GridFS - A file storage utility
 
 =head1 VERSION
 
-version v0.703.5
+version v0.999.998.1
 
 =head1 SYNOPSIS
 
@@ -585,11 +578,7 @@ Returns a list of the files in the database.
 
 =item *
 
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
+David Golden <david@mongodb.com>
 
 =item *
 
@@ -597,7 +586,11 @@ Mike Friedman <friedo@mongodb.com>
 
 =item *
 
-David Golden <david.golden@mongodb.org>
+Kristina Chodorow <kristina@mongodb.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
 
 =back
 
