@@ -19,9 +19,11 @@ package MongoDB::OID;
 # ABSTRACT: A Mongo Object ID
 
 use version;
-our $VERSION = 'v0.703.5'; # TRIAL
+our $VERSION = 'v0.999.998.1'; # TRIAL
 
+use MongoDB::BSON;
 use Moose;
+use MongoDB;
 use namespace::clean -except => 'meta';
 
 #pod =head1 NAME
@@ -49,7 +51,7 @@ use namespace::clean -except => 'meta';
 #pod     my $id3 = MongoDB::OID->new($id1->value);
 #pod     my $id4 = MongoDB::OID->new($id1);
 #pod
-#pod Now C<$id1>, C<$id2>, $<$id3> and C<$id4> will have the same value.
+#pod Now C<$id1>, C<$id2>, C<$id3> and C<$id4> will have the same value.
 #pod
 #pod OID generation is thread safe.
 #pod
@@ -72,9 +74,17 @@ has value => (
     is      => 'ro',
     isa     => 'Str',
     required => 1,
-    builder => 'build_value',
+    builder => '_build_value',
 );
 
+# XXX need to set up typedef with str length
+# msg: "OIDs need to have a length of 24 bytes"
+
+sub _build_value {
+    my ($self) = @_;
+    return MongoDB::BSON::generate_oid();
+}
+
 around BUILDARGS => sub {
     my $orig = shift;
     my $class = shift;
@@ -87,12 +97,6 @@ around BUILDARGS => sub {
     return $class->$orig(@_);
 };
 
-sub build_value {
-    my $self = shift;
-
-    _build_value($self, @_ ? @_ : ());
-}
-
 #pod =head1 METHODS
 #pod
 #pod =head2 to_string
@@ -171,7 +175,7 @@ MongoDB::OID - A Mongo Object ID
 
 =head1 VERSION
 
-version v0.703.5
+version v0.999.998.1
 
 =head1 SYNOPSIS
 
@@ -194,7 +198,7 @@ constructor.  For example:
     my $id3 = MongoDB::OID->new($id1->value);
     my $id4 = MongoDB::OID->new($id1);
 
-Now C<$id1>, C<$id2>, $<$id3> and C<$id4> will have the same value.
+Now C<$id1>, C<$id2>, C<$id3> and C<$id4> will have the same value.
 
 OID generation is thread safe.
 
@@ -253,11 +257,7 @@ C<{"$oid" : "012345678901234567890123"}>.
 
 =item *
 
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
+David Golden <david@mongodb.com>
 
 =item *
 
@@ -265,7 +265,11 @@ Mike Friedman <friedo@mongodb.com>
 
 =item *
 
-David Golden <david.golden@mongodb.org>
+Kristina Chodorow <kristina@mongodb.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
 
 =back
 
