@@ -20,7 +20,7 @@ package MongoDB::Timestamp;
 # ABSTRACT: Replication timestamp
 
 use version;
-our $VERSION = 'v0.703.5'; # TRIAL
+our $VERSION = 'v0.999.998.1'; # TRIAL
 
 #pod =head1 NAME
 #pod
@@ -80,7 +80,7 @@ MongoDB::Timestamp - Replication timestamp
 
 =head1 VERSION
 
-version v0.703.5
+version v0.999.998.1
 
 =head1 SYNOPSIS
 
@@ -109,11 +109,7 @@ Incrementing field.
 
 =item *
 
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
+David Golden <david@mongodb.com>
 
 =item *
 
@@ -121,7 +117,11 @@ Mike Friedman <friedo@mongodb.com>
 
 =item *
 
-David Golden <david.golden@mongodb.org>
+Kristina Chodorow <kristina@mongodb.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
 
 =back
 
