@@ -15,7 +15,6 @@
  */
 
 #include "perl_mongo.h"
-#include "mongo_link.h"
 
 #ifdef WIN32
 #include <memory.h>
@@ -36,7 +35,6 @@ static SV *bson_to_sv (bson_iter_t * iter, char *dt_type, int inflate_dbrefs, in
 static perl_mutex inc_mutex;
 #endif
 
-static int perl_mongo_inc = 0;
 int perl_mongo_machine_id;
 
 static SV *utf8_flag_on;
@@ -182,6 +180,10 @@ perl_mongo_call_function (const char *func, int num, ...) {
 }
 
 static void perl_mongo_regex_flags( char *flags_ptr, SV *re ) {
+  int ret_count;
+  SV *flags_sv;
+  SV *pat_sv;
+  char *flags;
   dSP;
   ENTER;
   SAVETMPS;
@@ -189,7 +191,7 @@ static void perl_mongo_regex_flags( char *flags_ptr, SV *re ) {
   XPUSHs (re);
   PUTBACK;
 
-  int ret_count = call_pv( "re::regexp_pattern", G_ARRAY );
+  ret_count = call_pv( "re::regexp_pattern", G_ARRAY );
   SPAGAIN;
 
   if ( ret_count != 2 ) { 
@@ -197,10 +199,10 @@ static void perl_mongo_regex_flags( char *flags_ptr, SV *re ) {
   }
 
   // regexp_pattern returns two items (in list context), the pattern and a list of flags
-  SV *flags_sv = POPs;
-  SV *pat_sv   = POPs;
+  flags_sv = POPs;
+  pat_sv   = POPs;
 
-  char *flags = SvPVutf8_nolen(flags_sv);
+  flags = SvPVutf8_nolen(flags_sv);
 
   strncpy( flags_ptr, flags, 7 );
 }
@@ -306,6 +308,40 @@ perl_mongo_construct_instance_with_magic (const char *klass, void *ptr, MGVTBL *
   return ret;
 }
 
+SV *
+perl_mongo_construct_instance_single_arg (const char *klass, SV *arg)
+{
+  dSP;
+  SV *ret;
+  I32 count;
+
+  ENTER;
+  SAVETMPS;
+
+  PUSHMARK (SP);
+  mXPUSHp (klass, strlen (klass));
+  XPUSHs(arg);
+  PUTBACK;
+
+  count = call_method ("new", G_SCALAR);
+
+  SPAGAIN;
+
+  if (count != 1) {
+    croak ("constructor didn't return an instance");
+  }
+
+  ret = POPs;
+  SvREFCNT_inc (ret);
+
+  PUTBACK;
+  FREETMPS;
+  LEAVE;
+
+  return ret;
+}
+
+
 static SV *bson_to_av (bson_iter_t * iter, char *dt_type, int inflate_dbrefs, int inflate_regexps, SV *client );
 
 static SV *
@@ -349,6 +385,10 @@ elem_to_sv (const bson_iter_t * iter, char *dt_type, int inflate_dbrefs, int inf
       str = bson_iter_utf8(iter, &len);
     }
 
+    if ( ! is_utf8_string((const U8*)str,len)) {
+      croak( "Invalid UTF-8 detected while decoding BSON" );
+    }
+
     // this makes a copy of the buffer
     // len includes \0
     value = newSVpvn(str, len);
@@ -437,7 +477,11 @@ elem_to_sv (const bson_iter_t * iter, char *dt_type, int inflate_dbrefs, int inf
 #if defined(MONGO_USE_64_BIT_INT)
     value = newSViv(bson_iter_int64(iter));
 #else
-    value = newSVnv((double)MONGO_64p(bson_iter_int64(iter)));
+    char buf[22];
+    sprintf(buf,"%" PRIi64,bson_iter_int64(iter));
+    load_module(0,newSVpvs("Math::BigInt"),NULL,NULL);
+    SV *as_str = sv_2mortal(newSVpv(buf,0));
+    value = perl_mongo_construct_instance_single_arg("Math::BigInt", as_str);
 #endif
     break;
   }
@@ -452,9 +496,11 @@ elem_to_sv (const bson_iter_t * iter, char *dt_type, int inflate_dbrefs, int inf
       // raw epoch
       value = newSViv(ms_i);
     } else if ( strcmp( dt_type, "DateTime::Tiny" ) == 0 ) {
+      time_t epoch;
+      struct tm *dt;
       datetime = sv_2mortal(newSVpv("DateTime::Tiny", 0));
-      time_t epoch = bson_iter_time_t(iter);
-      struct tm *dt = gmtime( &epoch );
+      epoch = bson_iter_time_t(iter);
+      dt = gmtime( &epoch );
 
       value = 
         perl_mongo_call_function("DateTime::Tiny::new", 13, datetime,
@@ -577,10 +623,11 @@ elem_to_sv (const bson_iter_t * iter, char *dt_type, int inflate_dbrefs, int inf
   case BSON_TYPE_CODE: {
     const char * code;
     uint32_t len;
-    
+    SV *code_sv;
+
     code = bson_iter_code(iter, &len);
 
-    SV * code_sv = sv_2mortal(newSVpvn(code, len));
+    code_sv = sv_2mortal(newSVpvn(code, len));
 
     value = perl_mongo_construct_instance("MongoDB::Code", "code", code_sv, NULL);
 
@@ -590,19 +637,19 @@ elem_to_sv (const bson_iter_t * iter, char *dt_type, int inflate_dbrefs, int inf
     const char * code;
     const uint8_t * scope;
     uint32_t code_len, scope_len;
-
-    code = bson_iter_codewscope(iter, &code_len, &scope_len, &scope);
-
-    SV * code_sv = sv_2mortal(newSVpvn(code, code_len));
-
+    SV * code_sv;
+    SV * scope_sv;
     bson_t bson;
     bson_iter_t child;
 
+    code = bson_iter_codewscope(iter, &code_len, &scope_len, &scope);
+    code_sv = sv_2mortal(newSVpvn(code, code_len));
+
     if ( ! ( bson_init_static(&bson, scope, scope_len) && bson_iter_init(&child, &bson) ) ) {
         croak("error iterating BSON type %d\n", bson_iter_type(iter));
     }
 
-    SV * scope_sv = bson_to_sv(&child, dt_type, inflate_dbrefs, inflate_regexps, client );
+    scope_sv = bson_to_sv(&child, dt_type, inflate_dbrefs, inflate_regexps, client );
     value = perl_mongo_construct_instance("MongoDB::Code", "code", code_sv, "scope", scope_sv, NULL);
 
     break;
@@ -677,10 +724,10 @@ perl_mongo_buffer_to_sv(buffer * buffer, char * dt_type, int inflate_dbrefs, int
 SV *
 perl_mongo_bson_to_sv (const bson_t * bson, char *dt_type, int inflate_dbrefs, int inflate_regexps, SV *client )
 {
+  bson_iter_t iter;
   utf8_flag_on = get_sv("MongoDB::BSON::utf8_flag_on", 0);
   use_binary = get_sv("MongoDB::BSON::use_binary", 0);
 
-  bson_iter_t iter;
   if ( ! bson_iter_init(&iter, bson) ) {
       croak( "error creating BSON iterator" );
   }
@@ -701,6 +748,11 @@ bson_to_sv (bson_iter_t * iter, char *dt_type, int inflate_dbrefs, int inflate_r
     SV *value;
 
     name = bson_iter_key(iter);
+
+    if ( ! is_utf8_string((const U8*)name,strlen(name))) {
+      croak( "Invalid UTF-8 detected while decoding BSON" );
+    }
+
     key_num++;
     /* check if this is a DBref. We must see the keys
        $ref, $id, and $db in that order, with no extra keys */
@@ -890,6 +942,11 @@ hv_to_bson (bson_t * bson, SV *sv, AV *ids, stackette *stack, int is_insert)
     if (!utf8) {
       key = (const char *) bytes_to_utf8((U8 *)key, &len);
     }
+
+    if ( ! is_utf8_string((const U8*)key,len)) {
+        croak( "Invalid UTF-8 detected while encoding BSON" );
+    }
+
     append_sv (bson, key, *hval, stack, is_insert);
     if (!utf8) {
       Safefree(key);
@@ -1040,11 +1097,11 @@ const char * clean_key(const char * str, int is_insert) {
   }
 
   if (is_insert && strchr(str, '.')) {
-    croak("inserts cannot contain the . character");
+    croak("documents for storage cannot contain the . character");
   }
 
   if (special_char && SvPOK(special_char) && SvPV_nolen(special_char)[0] == str[0]) {
-    char * out = strdup(str);
+    char * out = savepv(str);
 
     *out = '$';
 
@@ -1065,7 +1122,7 @@ append_sv (bson_t * bson, const char * in_key, SV *sv, stackette *stack, int is_
     }
     else {
       bson_append_null(bson, key, -1);
-      if (in_key != key) free((char *)key);
+      if (in_key != key) Safefree((char *)key);
       return;
     }
   }
@@ -1262,7 +1319,11 @@ append_sv (bson_t * bson, const char * in_key, SV *sv, stackette *stack, int is_
           croak("MongoDB::BSON::String must be a blessed string reference");
         }
 
-        str = SvPV(str_sv, str_len);
+        str = SvPVutf8(str_sv, str_len);
+
+        if ( ! is_utf8_string((const U8*)str,str_len)) {
+          croak( "Invalid UTF-8 detected while encoding BSON" );
+        }
 
         bson_append_utf8(bson, key, -1, str, str_len);
       }
@@ -1407,6 +1468,10 @@ append_sv (bson_t * bson, const char * in_key, SV *sv, stackette *stack, int is_
         STRLEN len;
         const char *str = SvPVutf8(sv, len);
 
+        if ( ! is_utf8_string((const U8*)str,len)) {
+          croak( "Invalid UTF-8 detected while encoding BSON" );
+        }
+
         bson_append_utf8(bson, key, -1, str, len);
       }
       break;
@@ -1417,32 +1482,33 @@ append_sv (bson_t * bson, const char * in_key, SV *sv, stackette *stack, int is_
     }
   }
 
-  if (in_key != key) free((char *)key);
+  if (in_key != key) Safefree((char *)key);
 }
 
 static void serialize_regex_obj(bson_t *bson, const char *key, 
                                 const char *pattern, const char *flags ) { 
   size_t pattern_length = strlen( pattern );
-  size_t flags_length   = strlen( flags );
+  char *buf;
 
-  char *buf = malloc( pattern_length + 1 ); 
-  memcpy( buf, pattern, pattern_length );
+  Newx(buf, pattern_length + 1, char );
+  Copy(pattern, buf, pattern_length, char );
   buf[ pattern_length ] = '\0';
   bson_append_regex(bson, key, -1, buf, flags);
-  free(buf);
+  Safefree(buf);
 }
 
 static void serialize_regex(bson_t * bson, const char *key, REGEXP *re, SV * sv) {
   char flags[]     = {0,0,0,0,0};
+  char * buf;
   serialize_regex_flags(flags, sv);
 
-  char * buf = malloc(RX_PRELEN(re) + 1);
-  memcpy(buf, RX_PRECOMP(re), RX_PRELEN(re));
+  Newx(buf, (RX_PRELEN(re) + 1), char );
+  Copy(RX_PRECOMP(re), buf, RX_PRELEN(re), char );
   buf[RX_PRELEN(re)] = '\0';
 
   bson_append_regex(bson, key, -1, buf, flags);
 
-  free(buf);
+  Safefree(buf);
 }
 
 static void serialize_regex_flags(char * flags, SV *sv) {
@@ -1516,7 +1582,7 @@ void perl_mongo_sv_to_buffer(buffer * buf, SV *sv, AV *ids)
   writer = bson_writer_new((uint8_t **)&buf->start, &buf_len, offset, &mongo_renew);
 
   bson_writer_begin(writer, &bson);
-  perl_mongo_sv_to_bson(bson, sv, ids);
+  perl_mongo_sv_to_bson(bson, sv, ids!=0, ids);
   bson_writer_end(writer);
 
   buf->end = buf->start + buf_len;
@@ -1526,8 +1592,7 @@ void perl_mongo_sv_to_buffer(buffer * buf, SV *sv, AV *ids)
 }
 
 void
-perl_mongo_sv_to_bson (bson_t * bson, SV *sv, AV *ids) {
-  int is_insert = ids != NO_PREP;
+perl_mongo_sv_to_bson (bson_t * bson, SV *sv, int is_insert, AV *ids) {
 
   if (!SvROK (sv)) {
     croak ("not a reference");
